-- DCS LUA SCRIPT FOR THE FLYPT MOVER
-- For more info, visit: https://www.xsimulator.net/community/threads/flypt-mover-interface.13464/
-- Copy this file to the folder %USER%/Saved Games/DCS/Scripts/Export.lua
-- If the Scripts folder doesn't exist, create it.

function LuaExportStart()
    package.path = package.path..";.\\LuaSocket\\?.lua"
    package.cpath = package.cpath..";.\\LuaSocket\\?.dll"
    socket = require("socket")
	
    host = host or "127.0.0.1"
	
	-- Send data to this port when flying
    airPort = airPort or 4123
    airClient = socket.udp()
    airClient:settimeout(0)
    airClient:setpeername(host, airPort)
	
	-- Send data to this port when grounded
	groundPort = groundPort or 4124
	groundClient = socket.udp()
    groundClient:settimeout(0)
    groundClient:setpeername(host, groundPort)
end

function LuaExportAfterNextFrame()

	local acceleration = LoGetAccelerationUnits()  
	local speed = LoGetVectorVelocity()
    local pitch, roll, yaw = LoGetADIPitchBankYaw()
    local rotationSpeed = LoGetAngularVelocity()
	local altitude = LoGetAltitudeAboveGroundLevel()
 	
	-- The FlyPT Mover uses Z for vertical and Y to the front
	-- That's the opposite in DCS
	-- Values sent in one string, separated by spaces
	
	-- Index 00 = LoGetAccelerationUnits().x = Longitudinal acceleration (G)
	-- Index 01 = LoGetAccelerationUnits().z = Lateral acceleration (G)
	-- Index 02 = LoGetAccelerationUnits().y = Vertical acceleration (G)
	-- Index 03 = LoGetVectorVelocity().x = Lateral speed (m/s)
	-- Index 04 = LoGetVectorVelocity().z = Longitudinal speed (m/s)
	-- Index 05 = LoGetVectorVelocity().y = Vertical speed (m/s)
	-- Index 06 = LoGetAngularVelocity().z = Rotation speed around z (yaw in rad/s)
	-- Index 07 = LoGetAngularVelocity().y = Rotation speed around y (roll in rad/s)
	-- Index 08 = LoGetAngularVelocity().x = Rotation speed around x (pitch in rad/s)
	-- Index 09 = LoGetADIPitchBankYaw(0) = Yaw position (rad)
	-- Index 10 = LoGetADIPitchBankYaw(1) = Roll position (rad)
	-- Index 11 = LoGetADIPitchBankYaw(2) = Pitch position (rad)
	-- Index 12 = LoGetTrueAirSpeed() = Air speed (m/s)
	-- Index 13 = LoGetAircraftDrawArgumentValue(1) = Front/Rear landing gear (0 to 1)?
	-- Index 14 = LoGetAircraftDrawArgumentValue(2) = Turning landing gear (0 to 1)?
	-- Index 15 = LoGetAircraftDrawArgumentValue(4) = Left landing gear (0 to 1)?
	-- Index 16 = LoGetAircraftDrawArgumentValue(6) = Right landing gear (0 to 1)?
	-- Index 17 = LoGetAltitudeAboveGroundLevel() = Vertical position relative to ground (m)
	-- Index 17 = LoGetModelTime() = To track frame sequence (not used right now, in seconds)
	
	-- If there's action on gear suspension, we are grounded else we are flying 
	-- Tests what I think is left and right gear train. The other one is the middle one and in planes where the weel stays outside, it seems to vibrate... (need more info about this and keys used)
	-- Can't use LoGetAltitudeAboveGroundLevel, because even on ground this value can be big. It depends on the airplane.
	
	
	-- ALTERNATIVE 1
	-- Faster
	-- Send data only to one client. 
	-- If touching ground in any wheel, send to ground, else send to air
	-- If you want to use this one, uncoment code and coment ALTERNATIVE 2 (use   --[[   and  --]]   to coment blocks of code)
	--
    if LoGetAircraftDrawArgumentValue(1)+LoGetAircraftDrawArgumentValue(4)+LoGetAircraftDrawArgumentValue(6)>0 then
        if groundClient then
            socket.try(groundClient:send(string.format("%.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f", acceleration.z, acceleration.x, acceleration.y, speed.x, speed.z, speed.y, rotationSpeed.y, rotationSpeed.x, rotationSpeed.z, yaw, roll, pitch, LoGetTrueAirSpeed(),LoGetAircraftDrawArgumentValue(1),LoGetAircraftDrawArgumentValue(2),LoGetAircraftDrawArgumentValue(4), LoGetAircraftDrawArgumentValue(6), LoGetAltitudeAboveGroundLevel(), LoGetModelTime())))
            socket.try(airClient:send("0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0"))
        end
    else
        if airClient then
            socket.try(airClient:send(string.format("%.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f", acceleration.z, acceleration.x, acceleration.y, speed.x, speed.z, speed.y, rotationSpeed.y, rotationSpeed.x, rotationSpeed.z, yaw, roll, pitch, LoGetTrueAirSpeed(),LoGetAircraftDrawArgumentValue(1),LoGetAircraftDrawArgumentValue(2),LoGetAircraftDrawArgumentValue(4), LoGetAircraftDrawArgumentValue(6), LoGetAltitudeAboveGroundLevel(), LoGetModelTime())))
            socket.try(groundClient:send("0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0"))
        end
    end	--
	
	-- ALTERNATIVE 2
	-- Slower
	-- Send to both clients, but value affected by amount of wheels touching ground
	-- If you want to use this one, uncoment code and coment ALTERNATIVE 1
	--[[
	local amountOfGround=0.0;
	if LoGetAircraftDrawArgumentValue(1)>0 then 
		amountOfGround=amountOfGround+0.3; -- 0.3, we are looking at 3 wheels
	end
	if LoGetAircraftDrawArgumentValue(4)>0 then 
		amountOfGround=amountOfGround+0.35; -- 0.35, we are looking at 3 wheels
	end
	if LoGetAircraftDrawArgumentValue(6)>0 then 
		amountOfGround=amountOfGround+0.35; -- 0.35, we are looking at 3 wheels
	end 
	local amountOfAir=1.0-amountOfGround;
	
	if groundClient then 
		socket.try(groundClient:send(string.format("%.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f", amountOfGround*(acceleration.x), amountOfGround*(acceleration.z), amountOfGround*(acceleration.y), amountOfGround*(speed.x), amountOfGround*(speed.z), amountOfGround*(speed.y), amountOfGround*(rotationSpeed.y), amountOfGround*(rotationSpeed.x), amountOfGround*(rotationSpeed.z), amountOfGround*yaw, amountOfGround*roll, amountOfGround*pitch, LoGetTrueAirSpeed(),LoGetAircraftDrawArgumentValue(1),LoGetAircraftDrawArgumentValue(2),LoGetAircraftDrawArgumentValue(4),LoGetAircraftDrawArgumentValue(6), LoGetAltitudeAboveGroundLevel(), LoGetModelTime())))
	end
	if airClient then 
		socket.try(airClient:send(string.format("%.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f %.4f", amountOfAir*(acceleration.x), amountOfAir*(acceleration.z), amountOfAir*(acceleration.y), amountOfAir*(speed.x), amountOfAir*(speed.z), amountOfAir*(speed.y), amountOfAir*(rotationSpeed.y), amountOfAir*(rotationSpeed.x), amountOfAir*(rotationSpeed.z), amountOfAir*yaw, amountOfAir*roll, amountOfAir*pitch, LoGetTrueAirSpeed(),LoGetAircraftDrawArgumentValue(1),LoGetAircraftDrawArgumentValue(2),LoGetAircraftDrawArgumentValue(4),LoGetAircraftDrawArgumentValue(6), LoGetAltitudeAboveGroundLevel(), LoGetModelTime())))
	end
	--]]
end

function LuaExportStop()
    if groundClient then
        groundClient:close()
    end
	if airClient then
        airClient:close()
    end
end
local SimShakerlfs=require('lfs'); dofile(SimShakerlfs.writedir()..'Scripts/SimShaker.lua')

dofile(lfs.writedir()..[[Scripts\SimShaker-export-core\ExportCore.lua]])
